VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsUSB_SPS"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ###########################################################################
' Name:         clsUSB_SPS
' Modultype:    classmodule
' Version:      1.0
' Datw:         2006-05-03
' Description:  This module interfaces the USB-SPS 5+3 board
'               with a FTDI FT245R chip
'
' This classmodule has the following properties:
'
' PROPERTY        |Get/Let| DESCRIPTION AND DEFAULTS [ ]
' -----------------------------------------------------------------------------------
' SerialNumber    |  G    | serialnumber from EEPROM (16 char)           (string)[""]
' In              |  G    | read state of the 5 digital inputs            (integer)[]
' Out             |  G/L  | state of the 3 digital outputs                (integer)[]
' -----------------------------------------------------------------------------------
'
'   The module provides the following methods:
'
'   Identify()
'       FTDI devices of 'USB-Opto I/O 5+3' type as enumerated by windows in infinite loop
'       if successful, property SerialNumber is set - otherwise it's empty
'       loop starts with device 0, and restarts with 0 after last device
'
'   Connect(Serial as string)as boolean
'       connects to the device with the given Serialnumber, true if successful
'
'###############################################################
' top view of interface-card terminal clamps and
' conneted datalines D0..D7 of the FT245 chip:
' _____________________________________________________________
' |      J4      |      J3      |      J2      |      J1      |
' | + 2 -  + 1 - | + 0 -  + 4 - | + 3 -  + 2 - | + 1 -  + 0 - |
' | A6-A5  A4-A3 | A2-A1 E10-E9 | E8-E7  E6-E5 | E4-E3  E2-E1 |
' |  D0      D4  |   D2     D1  |   D7    D5   |  D3      D6  |
' |                                                           |
' |                                                           |
' |                                                           |
' |                                                           |
' |                                                           |
' |                                                           |
' |                                                           |
' _____________________________________________________________
'
Option Explicit

Private ghdlDevice As Long

' use MProg to set device identifier
Private Const DEVICE_IDENT = "USB-Opto I/O 5+3"

'====================================================================
'APIGID32.DLL by DESAWARE Inc. (www.desaware.com), see Dan Appleman's
'"Visual Basic Programmer's Guide to the WIN32-API"; here used to get
'the addresses of the VB-bytearrays:
'====================================================================
Private Declare Function agGetAddressForObject Lib "apigid32.dll" (object As Any) As Long

'===========================
'FTDI INTERFACE
'===========================

Private Declare Function FT_Open Lib "FTD2XX.DLL" (ByVal intDeviceNumber As Long, ByRef lngHandle As Long) As Long
Private Declare Function FT_OpenEx Lib "FTD2XX.DLL" (ByVal arg1 As String, ByVal arg2 As Long, ByRef lngHandle As Long) As Long
Private Declare Function FT_Close Lib "FTD2XX.DLL" (ByVal lngHandle As Long) As Long

Private Declare Function FT_Write Lib "FTD2XX.DLL" (ByVal lngHandle As Long, ByVal lpszBuffer As String, ByVal lngBufferSize As Long, ByRef lngBytesWritten As Long) As Long
Private Declare Function FT_WriteByte Lib "FTD2XX.DLL" Alias "FT_Write" (ByVal lngHandle As Long, ByRef lpszBuffer As Any, ByVal lngBufferSize As Long, ByRef lngBytesWritten As Long) As Long
Private Declare Function FT_SetBaudRate Lib "FTD2XX.DLL" (ByVal lngHandle As Long, ByVal lngBaudRate As Long) As Long

Private Declare Function FT_GetBitMode Lib "FTD2XX.DLL" (ByVal lngHandle As Long, ByRef intData As Any) As Long
Private Declare Function FT_SetBitMode Lib "FTD2XX.DLL" (ByVal lngHandle As Long, ByVal intMask As Byte, ByVal intMode As Byte) As Long

Private Declare Function FT_SetLatencyTimer Lib "FTD2XX.DLL" (ByVal Handle As Long, ByVal pucTimer As Byte) As Long

'==============================================================
'Declarations for the EEPROM-accessing functions in FTD2XX.dll:
'==============================================================
Private Declare Function FT_EE_Read Lib "FTD2XX.DLL" (ByVal lngHandle As Long, ByRef lpData As FT_PROGRAM_DATA) As Long

Private Type FT_PROGRAM_DATA

    signature1 As Long                  '0x00000000
    signature2 As Long                  '0xFFFFFFFF
    version As Long                     '0

    VendorId As Integer                 '0x0403
    ProductId As Integer                '0x6001
    Manufacturer As Long                '32 "FTDI"
    ManufacturerID As Long              '16 "FT"
    Description As Long                 '64 "USB HS Serial Converter"
    SerialNumber As Long                '16 "FT000001" if fixed, or NULL
    MaxPower As Integer                 ' // 0 < MaxPower <= 500
    PnP As Integer                      ' // 0 = disabled, 1 = enabled
    SelfPowered As Integer              ' // 0 = bus powered, 1 = self powered
    RemoteWakeup As Integer             ' // 0 = not capable, 1 = capable
     'Rev4 extensions:
    Rev4 As Byte                        ' // true if Rev4 chip, false otherwise
    IsoIn As Byte                       ' // true if in endpoint is isochronous
    IsoOut As Byte                      ' // true if out endpoint is isochronous
    PullDownEnable As Byte              ' // true if pull down enabled
    SerNumEnable As Byte                ' // true if serial number to be used
    USBVersionEnable As Byte            ' // true if chip uses USBVersion
    USBVersion As Integer               ' // BCD (0x0200 => USB2)
    'FT2232C extensions:
    Rev5 As Byte                        'non-zero if Rev5 chip, zero otherwise
    IsoInA As Byte                      'non-zero if in endpoint is isochronous
    IsoInB As Byte                      'non-zero if in endpoint is isochronous
    IsoOutA As Byte                     'non-zero if out endpoint is isochronous
    IsoOutB As Byte                     'non-zero if out endpoint is isochronous
    PullDownEnable5 As Byte             'non-zero if pull down enabled
    SerNumEnable5 As Byte               'non-zero if serial number to be used
    USBVersionEnable5 As Byte           'non-zero if chip uses USBVersion
    USBVersion5 As Integer              'BCD 0x110 = USB 1.1, BCD 0x200 = USB 2.0
    AlsHighCurrent As Byte              'non-zero if interface is high current
    BlsHighCurrent As Byte              'non-zero if interface is high current
    IFAlsFifo As Byte                   'non-zero if interface is 245 FIFO
    IFAlsFifoTar As Byte                'non-zero if interface is 245 FIFO CPU target
    IFAlsFastSer As Byte                'non-zero if interface is Fast Serial
    AlsVCP As Byte                      'non-zero if interface is to use VCP drivers
    IFBlsFifo As Byte                   'non-zero if interface is 245 FIFO
    IFBlsFifoTar As Byte                'non-zero if interface is 245 FIFO CPU target
    IFBlsFastSer As Byte                'non-zero if interface is Fast Serial
    BlsVCP As Byte                      'non-zero if interface is to use VCP drivers
    'FT232R extensions
    UseExtOSC As Byte                   'non-zero use ext osc
    HighDriveIOs As Byte                'non-zero to use High Drive IO's
    EndPointSize As Byte                '64 Do not change
    PullDownEnableR As Byte             'non-zeero if pull down enabled
    SerNumEnableR As Byte               'non-zero if pull serial number enabled
    InvertTXD As Byte                   'non-zero if invert TXD
    InvertRXD As Byte                   'non-zero if invert RXD
    InvertRTS As Byte                   'non-zero if invert RTS
    InvertCTS As Byte                   'non-zero if invert CTS
    InvertDTR As Byte                   'non-zero if invert DTR
    InvertDSR As Byte                   'non-zero if invert DSR
    InvertDCD As Byte                   'non-zero if invert DCD
    InvertRI As Byte                    'non-zero if invert RI
    Cbus0 As Byte                       'Cbus Mux control
    Cbus1 As Byte                       'Cbus Mux control
    Cbus2 As Byte                       'Cbus Mux control
    Cbus3 As Byte                       'Cbus Mux control
    Cbus4 As Byte                       'Cbus Mux control
    RIsVCP As Byte                      'non-zero if using VCP driver
End Type

' Return codes
Private Const FT_OK = 0
Private Const FT_INVALID_HANDLE = 1
Private Const FT_DEVICE_NOT_FOUND = 2
Private Const FT_DEVICE_NOT_OPENED = 3
Private Const FT_IO_ERROR = 4
Private Const FT_INSUFFICIENT_RESOURCES = 5
Private Const FT_INVALID_PARAMETER = 6
Private Const FT_INVALID_BAUD_RATE = 7
Private Const FT_DEVICE_NOT_OPENED_FOR_ERASE = 8
Private Const FT_DEVICE_NOT_OPENED_FOR_WRITE = 9
Private Const FT_FAILED_TO_WRITE_DEVICE = 10
Private Const FT_EEPROM_READ_FAILED = 11
Private Const FT_EEPROM_WRITE_FAILED = 12
Private Const FT_EEPROM_ERASE_FAILED = 13
Private Const FT_EEPROM_NOT_PRESENT = 14
Private Const FT_EEPROM_NOT_PROGRAMMED = 15
Private Const FT_INVALID_ARGS = 16
Private Const FT_NOT_SUPPORTED = 17
Private Const FT_OTHER_ERROR = 18

Const WAIT_ABANDONED As Long = &H80
Const WAIT_FAILD As Long = &HFFFFFFFF
Const WAIT_OBJECT_0 As Long = &H0
Const WAIT_TIMEOUT As Long = &H102

' Flags for FT_ListDevices
Private Const FT_LIST_BY_NUMBER_ONLY = &H80000000
Private Const FT_LIST_BY_INDEX = &H40000000
Private Const FT_LIST_ALL = &H20000000

' Flags for FT_OpenEx
Private Const FT_OPEN_BY_SERIAL_NUMBER = 1
Private Const FT_OPEN_BY_DESCRIPTION = 2

Private miOutput As Long
Private mlDeviceNo As Long
Private mtDescription As String
Private mtSerialNumber As String

Public Property Get SerialNumber() As String
  SerialNumber = mtSerialNumber
End Property
' _____________________________________________________________
' |      J4      |      J3      |      J2      |      J1      |
' | + 2 -  + 1 - | + 0 -  + 4 - | + 3 -  + 2 - | + 1 -  + 0 - |
' | A6-A5  A4-A3 | A2-A1 E10-E9 | E8-E7  E6-E5 | E4-E3  E2-E1 |
' |  D0      D4  |   D2     D1  |   D7    D5   |  D3      D6  |
Public Property Get BitOut(ByVal Index As Integer) As Boolean
  Dim iMask As Integer
  Select Case Index
    Case 0
      iMask = &H1
    Case 1
      iMask = &H10
    Case 2
      iMask = &H4
    Case Else
      iMask = 0
  End Select
  BitOut = (miOutput And iMask) = 0
End Property
' _____________________________________________________________
' |      J4      |      J3      |      J2      |      J1      |
' | + 2 -  + 1 - | + 0 -  + 4 - | + 3 -  + 2 - | + 1 -  + 0 - |
' | A6-A5  A4-A3 | A2-A1 E10-E9 | E8-E7  E6-E5 | E4-E3  E2-E1 |
' |  D0      D4  |   D2     D1  |   D7    D5   |  D3      D6  |
Public Property Let BitOut(ByVal Index As Integer, ByVal b As Boolean)
  Dim iMask As Long
  Dim lWritten As Long, lRes As Long
  
  Select Case Index
    Case 0
      iMask = &H1
    Case 1
      iMask = &H10
    Case 2
      iMask = &H4
    Case Else
      iMask = 0
  End Select

  If b Then
    miOutput = miOutput And Not iMask
  Else
    miOutput = miOutput Or iMask
  End If
  lRes = FT_WriteByte(ghdlDevice, miOutput, 1, lWritten)
End Property
' _____________________________________________________________
' |      J4      |      J3      |      J2      |      J1      |
' | + 2 -  + 1 - | + 0 -  + 4 - | + 3 -  + 2 - | + 1 -  + 0 - |
' | A6-A5  A4-A3 | A2-A1 E10-E9 | E8-E7  E6-E5 | E4-E3  E2-E1 |
' |  D0      D4  |   D2     D1  |   D7    D5   |  D3      D6  |
Public Property Get BitInput(ByVal Index As Integer) As Boolean
  Dim iMask As Integer, iInput As Integer
  Dim lRes As Long
  
  Select Case Index
    Case 0
      iMask = &H40
    Case 1
      iMask = &H8
    Case 2
      iMask = &H20
    Case 3
      iMask = &H80
    Case 4
      iMask = &H2
    Case Else
      iMask = 0
  End Select
  lRes = FT_GetBitMode(ghdlDevice, iInput)
  BitInput = (iInput And iMask) <> 0
End Property

Public Sub Identify()
  'Bytearrays as "string-containers":
  Dim bDescription(64) As Byte
  Dim bSerialNumber(16) As Byte
  Dim fDone As Boolean
  
  Dim EEData As FT_PROGRAM_DATA
  Dim lRes As Long
  
  If ghdlDevice <> 0 Then
    lRes = FT_Close(ghdlDevice)
    mlDeviceNo = 0
  End If
  
  Do
    lRes = FT_Open(mlDeviceNo, ghdlDevice)
    If lRes = 0 Then
      EEData.Description = agGetAddressForObject(bDescription(0))
      EEData.SerialNumber = agGetAddressForObject(bSerialNumber(0))
      
      lRes = FT_EE_Read(ghdlDevice, EEData)
      If MakeString(bDescription) = DEVICE_IDENT Then
        mtSerialNumber = MakeString(bSerialNumber)
        fDone = True
      End If
      lRes = FT_Close(ghdlDevice)
      ghdlDevice = 0
      mlDeviceNo = mlDeviceNo + 1
    Else
      mtSerialNumber = ""
      fDone = mlDeviceNo = 0
      mlDeviceNo = 0
    End If
  Loop Until fDone
  
End Sub

Public Function Connect(Serial As String) As Boolean
  Dim lRes As Long
  Dim bSerialNumber(16) As Byte
  Dim EEData As FT_PROGRAM_DATA

  lRes = FT_OpenEx(Serial, FT_OPEN_BY_SERIAL_NUMBER, ghdlDevice) ' open of a specific device
  
  If lRes = 0 Then
    Connect = True
    EEData.SerialNumber = agGetAddressForObject(bSerialNumber(0))
    
    lRes = FT_EE_Read(ghdlDevice, EEData)
    mtSerialNumber = MakeString(bSerialNumber)
    
    lRes = FT_SetBitMode(ghdlDevice, &H15, &H1) ' set 1 for each output, $H1 for bitbang
    lRes = FT_SetBaudRate(ghdlDevice, 100000)
    lRes = FT_SetLatencyTimer(ghdlDevice, 2)
  Else
    Connect = False
  End If

End Function

Private Function MakeString(v As Variant) As String
  Dim t As String
  Dim i As Integer
  
  t = ""
  For i = 0 To UBound(v)
    If v(i) = 0 Then Exit For
    t = t & Chr(v(i))
  Next i
  MakeString = t
End Function

Private Sub Class_Terminate()
  Dim lRes As Long
  If ghdlDevice <> 0 Then
    lRes = FT_Close(ghdlDevice)
  End If
End Sub
